#!/bin/bash

# This script looks for a BOOST installation in a standard location.
# If a single BOOST installation is found, it prints out the full path of
# the dir containing subdir "boost" and returns with exit code 0.
# If none is found or several are found, it prints out an error message
# and returns with a non-zero exit code.

# When a single directory is found, only a few basic sanity checks are performed.

##################################################################
# List of most common directories in which boost header dir is normally found.
STD_BOOST_HPPDIRS="/usr/include:/usr/local/include:/opt/local/include:/sw/include:/usr/sfw/include:"

IFS=":"
HPPBOOSTPATHS=hppboost-paths
/bin/rm -rf $HPPBOOSTPATHS
for dir in $STD_BOOST_HPPDIRS
do
  if [ -d "$dir" -a -r "$dir" -a -d "$dir/boost" -a -r "$dir/boost" ]
  then
    echo "$dir" >> $HPPBOOSTPATHS
  fi
# 20110714 JAA replaced find command with much faster (but simpler) check above
#    find  "$dir"  -type d  -name boost  -print >> $HPPBOOSTPATHS  2> /dev/null
done

if [ \! -s $HPPBOOSTPATHS ]
then
  # Did not find any plausible BOOST installation, so return empty handed.
  echo "No BOOST installation found; looked inside $STD_BOOST_HPPDIRS"
  exit 1
fi

# Slightly odd call to wc is to avoid it printing out the file name.
if [ `wc -l < $HPPBOOSTPATHS` -ne 1 ]
then
  echo "*** Found multiple BOOST libraries ***"
  cat $HPPBOOSTPATHS
  /bin/rm $HPPBOOSTPATHS
  exit 2
fi

# We have found a single suitable directory (it exists & is readable,
# and contains readable subdir called "boost", so return it.
BOOST_HDR=`cat $HPPBOOSTPATHS`
/bin/rm $HPPBOOSTPATHS
echo "$BOOST_HDR"
