#!/bin/bash

# This script looks for the necessary BOOST sub-libraries.
# If found, it prints out the full path and returns with exit code 0.
# If not, it prints out a warning and returns with exit code 0.

##################################################################
if [ "$#" != "1" ]
then
  echo "$0: ERROR: requires 1 arg (full path of BOOST_HDR_DIR)"
  exit 1
fi

BOOST_HDR_DIR="$1"

# Special handling if BOOST is not fully installed...
if [ -d "$BOOST_HDR_DIR/stage" -a -r "$BOOST_HDR_DIR/stage" ]
then
  BOOST_LIB_DIR="$BOOST_HDR_DIR/stage/lib"
else
  BOOST_LIB_DIR="`dirname $BOOST_HDR_DIR`/lib"
fi
if [ \! -f "$BOOST_LIB_DIR/libboost_thread.a" ]
then
  echo "WARNING: libboost_thread.a not found in directory $BOOST_LIB_DIR"
  echo "(necessary only for GUI compilation)"
  exit 1
fi

echo "$BOOST_LIB_DIR"
exit 0
