//   Copyright (c)  2006-2010  Anna Bigatti, Massimo Caboara

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.


#include "CoCoA/DynamicBitset.H"
#include "CoCoA/SparsePolyRing.H"
#include "CoCoA/io.H" // for printing vectors
//#include "CoCoA/PPMonoid.H"

#include <algorithm>
using std::transform;
//#include <bitset>
using std::bitset;
#include <functional>
//??
#include <iterator>
using std::back_inserter;  // for transform
#include <iostream>
//using std::ostream;
//#include <list>
using std::list;
//#include <vector>
using std::vector;

//static const bool MAX_DEBUG = false;

namespace CoCoA
{
  DynamicBitset::OutputStyle DynamicBitset::ourOutputStyle; // clean;

// class DynamicBitset


  void DynamicBitset::myResize(long n) // only for ctors
  {
    CoCoA_ASSERT(n >= 0);
    myLenValue = n;
    if (n == 0) return;
    myVec.resize((n-1)/ourNumBitsInBlock + 1); // all 0s
  }
  

  DynamicBitset::DynamicBitset(ConstRefPPMonoidElem pp)
  {
    vector<long> v(NumIndets(owner(pp)));
    exponents(v, pp);
    // same as DynamicBitset(v)
    long n = len(v);
    myResize(n);
    for (long i=0; i!=n; ++i)
      if (v[i] != 0)  mySet(i);
  }


//   DynamicBitset::DynamicBitset(const vector<long>& v)
//   {
//     myResize(v.size());
//     for (long i=0; i!=v.size(); ++i)
//       if (v[i] != 0)  mySet(i);
//   }


  DynamicBitset::DynamicBitset(long n)
  {
    if (n < 0) CoCoA_ERROR(ERR::NotNonNegative, "DynamicBitset(long n)");
    myResize(n);
  }


  DynamicBitset::DynamicBitset(const DynamicBitset& rhs)
  {
    myLenValue = rhs.myLenValue;
    myVec = rhs.myVec;
  }


  // Anna 15Apr2010:
  // if ">>=" appears to be too slow make vector of precomputed masks (static)
  bool DynamicBitset::IamAll1s() const
  {
    BitBlock mask = ~0; // 111111111
    for (vector<BitBlock>::const_iterator it=myVec.begin(); it!=myVec.end(); ++it)
      if ((std::operator^(*it, mask)).any())  // xor for bitset
      {
        if (it+1 != myVec.end() || myLenValue%ourNumBitsInBlock==0) return false;
        mask >>= (ourNumBitsInBlock - myLenValue%ourNumBitsInBlock); // 00000111111
        if (std::operator^(*it, mask).any()) return false;
      }
    return true;
  }
  

  void DynamicBitset::myOutputSelf(std::ostream& out) const
  {
    if (myLenValue==0)  return;
    vector<BitBlock>::const_reverse_iterator rit=myVec.rbegin();
    // first block without spurious 0s
    for (long n=(myLenValue-1)%ourNumBitsInBlock; n>=0; --n)
      out << rit->test(n);
    //    for (++rit; rit!=myVec.rend(); ++rit)  out << '-' << *rit;
    for (++rit; rit!=myVec.rend(); ++rit) out << *rit;
  }


  void DynamicBitset::myOutputSelf8(std::ostream& out) const
  {
    if (myLenValue==0)  return;
    vector<BitBlock>::const_reverse_iterator rit=myVec.rbegin();
    // first block without spurious 0s
    for (long n=(myLenValue-1)%ourNumBitsInBlock; n>=0; --n)
    {
      out << rit->test(n);
      if (n%8==0 && n!=0)  out << '.';
    }
    for (++rit; rit!=myVec.rend(); ++rit)
    {
      out << '-';
      for (long n=ourNumBitsInBlock-1; n>=0; --n)
      {
        out << rit->test(n);
        if (n%8==0 && n!=0)  out << '.';
      }
    }
  }


  namespace // Anna 15Apr2010: maybe not the best way.  Just practising STL.
  {
    inline unsigned long ToULong(const DynamicBitset::BitBlock& b)
    { return b.to_ulong(); }
  }
  
  void DynamicBitset::myOutputSelfLong(std::ostream& out) const
  {
    vector<unsigned long> v;
    v.reserve(myVec.size());
    transform(myVec.rbegin(), myVec.rend(), back_inserter(v), ToULong);
    out << v;
  }


  std::ostream& operator<<(std::ostream& out, const DynamicBitset& DB)
  {
    switch (DynamicBitset::ourOutputStyle)
    {
    case DynamicBitset::clean:          DB.myOutputSelf(out);  break;
    case DynamicBitset::AsRevVecOfLong: DB.myOutputSelfLong(out);  break;
    case DynamicBitset::WithSeparators: DB.myOutputSelf8(out);  break;
    default: CoCoA_ERROR(ERR::SERIOUS, "operator<<(ostream, DynamicBitset)");
    }
    return out;
  }


  // can be improved making vector of exponents (for dense PPMonoid)
  PPMonoidElem NewPP(const PPMonoid& PPM, const DynamicBitset& b)
  {
    if (NumIndets(PPM) != len(b))  CoCoA_ERROR(ERR::MixedSizes, "NewPP");
    PPMonoidElem pp(PPM);
    for (long i=0; i!=len(b); ++i)
      if (b.Iam1At(i))  pp *= indet(PPM, i);
    return pp;
  }


}// end namespace cocoa





/*

Some future optimization:

for IsFLesser: proceed in the test word by word:
compute the first word of g1-f, the first word of g2-f, the check

ConnectionBlock: use ptr and not iterators.

sparse representation for facets? Use vector, (sort?).
Only reasonable if density is much much lower than #VARS!

*/

// RCS header/log on the next few lines
// $Header: /Volumes/Home_1/cocoa/cvs-repository/CoCoALib-0.99/src/AlgebraicCore/DynamicBitset.C,v 1.11 2012/01/26 16:53:04 bigatti Exp $
// $Log: DynamicBitset.C,v $
// Revision 1.11  2012/01/26 16:53:04  bigatti
// -- added #include <iterator>
//
// Revision 1.10  2011/03/11 16:50:31  bigatti
// -- changed  unsigned int --> long
//
// Revision 1.9  2011/03/11 12:36:21  bigatti
// -- changed size_t --> long
// -- changed size --> len
//
// Revision 1.8  2010/05/28 15:50:42  bigatti
// -- cleaning
// -- commented out ctor taking a list of long
// -- moved some "facet" functions to TmpIsTree
//
// Revision 1.7  2010/04/27 16:08:58  bigatti
// -- changed error messages
//
// Revision 1.6  2010/04/21 11:35:55  bigatti
// -- changed Iam1InPos --> Iam1At
//
// Revision 1.5  2010/04/21 09:43:49  bigatti
// -- added static member field for choosing printing style
//
// Revision 1.4  2010/04/17 16:47:29  bigatti
// -- added ctor with ConstRefPPMonoidElem
// -- added include <algorithm>
// -- more consistent myOutputSelf* functions
//
// Revision 1.3  2010/04/15 16:01:39  bigatti
// -- DynamicBitset going towards final version
//
// Revision 1.2  2010/04/13 15:30:25  bigatti
// -- reorganized into (almost) final shape
//
// Revision 1.1  2010/03/30 15:20:43  bigatti
// -- was "facet" in TmpIsTree
//
