//   Copyright (c) 2009 Giovanni Lagorio (lagorio@disi.unige.it)
//
//   This file is part of the source of CoCoALib, the CoCoA Library.
//
//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.

#include <iostream>
#include <boost/tr1/memory.hpp>
#include <cstring>
#include <cctype>
#include <boost/iostreams/filter/newline.hpp>
#include "LineProviders.H"

namespace CoCoA {
namespace LexerNS {

using namespace std;
using namespace boost;
using namespace boost::iostreams;
using namespace CoCoA;

ReferenceCountedObject::~ReferenceCountedObject() {}

FileLineProvider::FileLineProvider(const std::string &filename) :
	filename(filename)
{
	input.push(newline_filter(newline::posix));
	const file_source fs(filename.c_str());
	if (!fs.is_open())
		throw IOException("Cannot open file \""+filename+"\" for reading.");
	input.push(fs);
}

string InteractiveLineProvider::promptSuffix("#");

string InteractiveLineProvider::prompt(const LexerStatus &ls, const ParserNS::ParserStatus &ps) {
	if (ps.isInRecoveryMode())
		throw ParserNS::AskingForNewInteractiveInputDuringRecoveryException();
	string s = ps.promptHeader();
	if (ps.isInTheMiddleOfAStatement())
		s.append("... ; ");
	else if (ls.isInMultiLineComment())
		s.append("/* ... ");
	else if (ls.isInStringLiteral())
		s.append("\" ... ");
	s.append(promptSuffix);
	s.append(" ");
	return s;
}

bool GetlineLineProvider::doReadNextLine(const LexerStatus &ls, const ParserNS::ParserStatus &ps, string &chars) {
	if (cin.eof())
		return false;
	cout << prompt(ls, ps) << flush;
	getline(cin, chars);
	return true;
}

bool FileLineProvider::doReadNextLine(const LexerStatus &, const ParserNS::ParserStatus &, string &chars) {
	if (input.eof())
		return false;
	getline(input, chars);
	if (input.bad())
		throw IOException("Cannot read from \""+filename+"\".");
#ifdef C5IDE
	this->wholeFile += chars;
	this->wholeFile += '\n';
#endif // #ifdef C5IDE
	return true;
}

} // namespace LexerNS
} // namespace CoCoA
