#! /bin/bash

# This script builds the CoCoA-5 GUI; it assumes that CoCoAInterpreter
# has already been built, and that the Qt libraries have been installed
# (and that the qmake command is available).

if [ $# -ne 5 ]
then
  echo "ERROR: $0 requires 5 args (full paths of BOOST_INC_DIR,  BOOST_LIB_DIR, libgmpxx, libgmp, gmp.h)"  > /dev/stderr
  exit 1
fi

BOOST_INC_DIR=$1
BOOST_LIB_DIR=$2
GMPXX_LIB=$3
GMP_LIB=$4
GMP_INC_DIR=$5

if [ \! -d "$BOOST_INC_DIR" -o \! -r "$BOOST_INC_DIR" ]
then
  echo "ERROR: $0 failed because cannot access BOOST in directory $BOOST_INC_DIR"  >/dev/stderr
  exit 1
fi
if [ \! -d "$BOOST_LIB_DIR" -o \! -f "$BOOST_LIB_DIR/libboost_thread.a" ]
then
  echo "ERROR: $0 failed because cannot find libboost_thread.a in directory $BOOST_LIB_DIR"  >/dev/stderr
  exit 1
fi
BOOST_LIBS="$BOOST_LIB_DIR/libboost_thread.a"

if [ \! -f "$GMPXX_LIB" ]
then
  echo "ERROR: $0 failed because $GMPXX_LIB is not a file" >/dev/stderr
fi
if [ \! -f "$GMP_LIB" ]
then
  echo "ERROR: $0 failed because $GMPXX_LIB is not a file" >/dev/stderr
fi
if [ \! -d "$GMP_INC_DIR" -o \! -r "$GMP_INC_DIR" ]
then
  echo "ERROR: $0 failed because cannot access GMP in directory $GMP_INC_DIR"  >/dev/stderr
  exit 1
fi


which qmake >/dev/null
if [ $? -ne 0 ]
then
  echo "ERROR: $0 failed because it cannot find qmake" >/dev/stderr
  exit 1
fi

##################################################################
# Set the variable ARCH if we are on a MacOS X system...

SYS_TYPE=`uname`

# On MacOSX we must explicitly state the "architecture".
# The block below should pick the right one (as dictated by libgmp).
if [ "$SYS_TYPE" = "Darwin" ]
then
  ARCH=`arch`  # either "i386" or "ppc"
  if [ "$ARCH" = "i386" ]
  then
    (cd ../../examples; make ex-empty) > /dev/null 2>&1
    arch -x86_64 ../../examples/ex-empty > /dev/null 2>&1
    if [ $? = 0 ]
    then
      ARCH=x86_64
    fi
  fi
  if [ "$ARCH" = "ppc" ]
  then
    (cd ../../examples; make ex-empty) > /dev/null 2>&1
    arch -ppc64 ../../examples/ex-empty > /dev/null 2>&1
    if [ $? = 0 ]
    then
      ARCH=ppc64
    fi
  fi
fi


##################################################################
# Generate C5.pro file

/bin/rm -f C5.pro

echo "## This file was generated automatically by $0"  > C5.pro
echo                                                  >> C5.pro
echo "INCLUDEPATH += $BOOST_INC_DIR ../../include"    >> C5.pro
echo "INCLUDEPATH += $GMP_INC_DIR"                    >> C5.pro
echo "LIBS += ../../lib/libcocoa.a"                   >> C5.pro
echo "LIBS += \"$BOOST_LIBS\""                        >> C5.pro
echo "LIBS += \"$GMPXX_LIB\""                         >> C5.pro
echo "LIBS += \"$GMP_LIB\""                           >> C5.pro

if [ "$SYS_TYPE" = "Darwin" ]
then
  echo "CONFIG += $ARCH    #MacOSX architecture"      >> C5.pro
fi
cat C5.pro.in                                         >> C5.pro


if [ "$SYS_TYPE" = "Darwin" ]
then
  DARWIN="-spec macx-g++"
fi
qmake $DARWIN C5.pro -o C5Makefile
if [ "$?" -ne 0 ]
then
  echo "ERROR: $0 failed because qmake failed for C5.pro"  >/dev/stderr
  exit 6
fi
